#ifndef _ELMO_INTERFACE_H_
#define _ELMO_INTERFACE_H_
/*
  Public definitions for the Elmo driver
*/

#include <linux/ioctl.h>

typedef struct {
    unsigned long  address;
    unsigned long  length;
    void          *data;
} ElmoIODescriptor;

typedef struct {
    unsigned long  length;
    void          *data;
} ElmoPLDDescriptor;

typedef struct {
  unsigned long address;
  unsigned short mask;
  unsigned short bits;
} ElmoMaskDescriptor;

#define kPLDMaxImageSize 0x20000

#undef DUMMY_ZERO_COPY_IO

#ifdef DUMMY_ZERO_COPY_IO

#define SGAT_DIO_DXFER_TO_DEV (-1)   
#define SGAT_DIO_DXFER_FROM_DEV (-2) 

typedef struct sgat_dio_hdr
{
    int dxfer_dir;        /* [i] data transfer direction  */
    unsigned short iovec_count; /* [i] 0 implies no scatter gather */
    unsigned int dxfer_len;     /* [i] byte count of data transfer */ 
    unsigned int timeout; /* [i] MAX_UINT->no timeout (unit: millisec) */
    unsigned short xferId;	
    int            status;      
    struct iovec  * sglp ;              /* [i] points to user scatter gather list */
} Sgat_dio_hdr_t;
#else
#include <sgat_dio.h>
#endif /* DUMMY_ZERO_COPY_IO */

typedef enum {
    ELMO_ERROR_HANG               =     0x00000001,
    ELMO_ERROR_TIMEOUT            =     0x00000002,
    ELMO_ERROR_ABORT              =     0x00000004,
    ELMO_ERROR_ATA_ABORT          =     0x00000008,
    ELMO_ERROR_UNKNOWN            =     0x00000010,
    ELMO_ERROR_SGAT_DIO           =     0x00000020,
    ELMO_ERROR_BULK_WORKER_INIT   =     0x00000040,
    ELMO_ERROR_ATA                =     0x00000080,


    /* End of list */
    ELMO_ERROR_END
}elmo_error_codes_e;

// ioctl selectors

#define     kElmoReadBridge                _IOW('G', 0, ElmoIODescriptor)
#define     kElmoWriteBridge               _IOW('G', 1, ElmoIODescriptor)
#define     kElmoReadWindow                _IOW('G', 2, ElmoIODescriptor)
#define     kElmoWriteWindow               _IOW('G', 3, ElmoIODescriptor)
#define     kElmoDownloadPLD               _IOW('G', 4, ElmoPLDDescriptor)
#define     kElmoInitDone                  _IO('G', 5)
#define     kElmoSetDebugMode              _IOR('G', 6, unsigned long)
#define     kElmoResetFIFOs                _IO('G', 7)
#define     kElmoReserve                   _IO('G', 8)
#define     kElmoRelease                   _IO('G', 9)
#define     kElmoReadFifoReset             _IO('G', 10)
#define     kElmoReportDiagMode            _IOR('G', 11, unsigned long)
#define     kElmoWriteBridgeMasked         _IOW('G', 12, ElmoMaskDescriptor)
#define     kElmoResetESS                  _IO('G', 13)
#define     kElmoDirectIO                  _IOW('G', 14, Sgat_dio_hdr_t)
#define     kElmoHasTei                    _IOW('G',15, unsigned long)
#define     kElmoGetEbiClkSpeed            _IOW('G',16, unsigned long)
#define     kElmoDisableWatchDog           _IO('G',17)
#define     kElmoEnableWatchDog            _IO('G',18)
#define     kElmoDirectIOAsync             _IOW('G', 19, Sgat_dio_hdr_t)
#define     kElmoCompleteDirectIOAsync     _IO('G', 20)
#define     kElmoCancelDirectIOAsync       _IO('G', 21)

// Exported functions for other drivers to use

#ifdef __KERNEL__

int ElmoReadBridge(unsigned long address, unsigned long length, void *data );
int ElmoWriteBridge(unsigned long address, unsigned long length, void *data );
int ElmoWriteBridgeMasked(unsigned long address, unsigned short mask, unsigned short bits );

#endif /* __KERNEL__ */


#endif /* _ELMO_INTERFACE_H_ */
